// ======================================================================
//
// DebugPage.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "DebugPage.h"

#if USE_DEBUG_PAGE

#include "ExitChain.h"

#include <string.h>
#include <stdio.h>
#include <stdarg.h>

// ======================================================================

DebugPage *DebugPageManager::firstPage;
DebugPage *DebugPageManager::currentPage;

// ======================================================================

DebugPage::DebugPage(int pageId, int pageWidth, int pageHeight)
{
	id      = pageId;
	width   = pageWidth;
	height  = pageHeight;
	text    = NEW char[width * height + 1];
	next    = NULL;

	text[width * height] = '\0';
	clearScreen();

	printf("page %d %d %d\nsecond test\n", pageId, pageWidth, pageHeight);
}

// ----------------------------------------------------------------------

DebugPage::~DebugPage(void)
{
	delete [] text;
}

// ----------------------------------------------------------------------

void DebugPage::clearScreen(void)
{
	memset(text, ' ', sizeof(*text)*width*height);
	cursorX = 0;
	cursorY = 0;
}

// ----------------------------------------------------------------------

void DebugPage::gotoXY(int x, int y)
{
	cursorX = clamp(0, x, width-1);
	cursorY = clamp(0, y, height-1);
}

// ----------------------------------------------------------------------

void DebugPage::whereXY(int *x, int *y) const
{
	if (x)
		*x = cursorX;
	if (y)
		*y = cursorY;
}

// ----------------------------------------------------------------------

void DebugPage::printf(const char *fmt, ...)
{
	va_list  va;
	char     buffer[256];
	char    *b;

	// format the buffer
	va_start(va, fmt);
	vsprintf(buffer, fmt, va);	
	va_end(va);

	// copy the formatted buffer into the output;
	for (b = buffer; *b; b++)
		if (*b == '\n')
		{
			cursorX = 0;
			if (++cursorY >= height)
				cursorY = height - 1;
		}
		else
		{
			text[(cursorY * width) + cursorX] = *b;
			if (++cursorX >= width)
			{
				cursorX = 0;
				if (++cursorY >= height)
					cursorY = height - 1;
			}	
		}
}

// ----------------------------------------------------------------------

void DebugPage::display(void) const
{
#if 1
	int  x, y, lastChar, skippedLines;
	static char buffer[96];

	for (skippedLines = 0, y = 0; y < height; y++)
	{
		// copy the line of text into the temp buffer		
		for (lastChar = 0, x = 0; x < width; x++)
		{
			buffer[x] = text[(y * width) + x];
			if (buffer[x] != ' ')
				lastChar = x + 1;
		}

		// check for anything on the line
		if (lastChar)
		{
			// write out any lines we compressed out to keep spacing correct
			while (skippedLines)
			{
				DPRINT_E_DPAGE(("\n"));
				skippedLines--;
			}

			// add a newline if the output stream is not going to wrap
			if (lastChar < 80)
				buffer[lastChar++] = '\n';

			// terminate the buffer
			buffer[lastChar] = '\0';

			// display the buffer
			DPRINT_E_DPAGE(("%s", buffer));
		}
		else
		{
			// compress out blank lines for the moment
			skippedLines++;
		}
	}
#else
	DPRINT_E_DPAGE(("%s", text));
#endif
}

// ======================================================================

void DebugPageManager::install(void)
{
	firstPage = NULL;
	currentPage = NULL;
	ExitChain::add(DebugPageManager::remove, "debug page", 0);
}

// ----------------------------------------------------------------------

void DebugPageManager::remove(void)
{
	DebugPage *page;

	while (firstPage)
	{
		// remove the first page from the list
		page = firstPage;
		firstPage = firstPage->next;

		// delete the first page
		delete page;
	}

	firstPage   = NULL;
	currentPage = NULL;
}
	
// ----------------------------------------------------------------------

DebugPage *DebugPageManager::createPage(int pageId, int width, int height)
{
	DebugPage *newPage;

	// make sure the page id is unique
	newPage = findPage(pageId);
	DEBUG_FATAL(newPage, ("DebugPageManager::createPage %d exists", pageId));

	// create the page
	newPage = NEW DebugPage(pageId, width, height);
	newPage->next = firstPage;

	// link the page in
	firstPage = newPage;

	if (!currentPage)
		currentPage = firstPage;

	return newPage;
}

// ----------------------------------------------------------------------

void DebugPageManager::deletePage(DebugPage *page)
{
	if (page)
	{
		DebugPage *back, *front;

		// linear search for the page
		for (back = NULL, front = firstPage; front && front != page; back = front, front = front->next)
			;

		DEBUG_FATAL(!front, ("DebugPageManager::deletePage missing"));

		// remove the page from the linked list
		if (back)
			back->next = front->next;
		else
			firstPage = front->next;

		if (page==currentPage)    // oops! don't let us delete what we're watching, or nasty fatals occur
			previousOutputPage();

		// free the page
		delete page;
	}
}

// ----------------------------------------------------------------------

DebugPage *DebugPageManager::findPage(int pageId)
{
	DebugPage *page;

	for (page = firstPage; page && page->id != pageId; page = page->next)
		;

	return page;
}

// ----------------------------------------------------------------------

void DebugPageManager::nextOutputPage(void)
{
	if (currentPage && currentPage->next)
		currentPage = currentPage->next;
	else
		currentPage = firstPage;
}

// ----------------------------------------------------------------------

void DebugPageManager::previousOutputPage(void)
{
	DebugPage *back, *front;

	if (currentPage && firstPage != currentPage)
	{
		for (back = NULL, front = firstPage; front && front != currentPage; back = front, front = front->next)
			;

		currentPage = back;
	}
	else
	{
		currentPage = firstPage;
		if (currentPage)
			while (currentPage->next)
				currentPage = currentPage->next;
	}

}

// ----------------------------------------------------------------------

void DebugPageManager::display(void)
{
	if (currentPage)
		currentPage->display();
}

// ----------------------------------------------------------------------

int DebugPageManager::findFreePage(void)
{
	int ret=0;

	for (;;)
		if (findPage(ret)!=NULL)
			ret++;
		else 
			return ret;
}

// ======================================================================

#endif
